use based::{auth::User, get_pg};

use super::Video;

pub trait VideoHistory {
    async fn last_video(&self) -> Option<uuid::Uuid>;
    async fn insert_history(&self, vid: uuid::Uuid);
    async fn history_of(&self, n: i64) -> Vec<Video>;
}

impl VideoHistory for User {
    async fn last_video(&self) -> Option<uuid::Uuid> {
        let res: Option<(uuid::Uuid,)> = sqlx::query_as("SELECT video_id FROM video_history WHERE username = $1 ORDER BY timestamp DESC LIMIT 1")
        .bind(&self.username)
        .fetch_optional(get_pg!())
        .await.unwrap();

        res.map(|x| x.0)
    }

    async fn insert_history(&self, vid: uuid::Uuid) {
        if let Some(prev) = self.last_video().await {
            if prev == vid {
                return;
            }
        }

        sqlx::query("INSERT INTO video_history (username, video_id) VALUES ($1, $2)")
            .bind(&self.username)
            .bind(vid)
            .execute(get_pg!())
            .await
            .unwrap();
    }

    async fn history_of(&self, n: i64) -> Vec<Video> {
        sqlx::query_as("SELECT v.* FROM video_history vh JOIN videos v ON vh.video_id = v.id WHERE vh.username = $1 ORDER BY vh.timestamp DESC LIMIT $2")
        .bind(&self.username)
        .bind(n)
        .fetch_all(get_pg!())
        .await.unwrap()
    }
}
