use std::process::{Command, Stdio};

pub fn extract_video_thumbnail(path: &str, time: f64) -> Vec<u8> {
    let output = Command::new("ffmpeg")
        .args([
            "-i",
            path, // Input video file path
            "-ss",
            &format!("{}", time), // Seek to the specific time
            "-vframes",
            "1", // Extract a single frame
            "-f",
            "image2", // Specify the image format
            "-",      // Output to stdout
        ])
        .stdout(Stdio::piped())
        .stderr(Stdio::piped())
        .output();

    match output {
        Ok(result) => {
            if !result.status.success() {
                log::error!(
                    "thumbnail_extraction:: ffmpeg error: {}",
                    String::from_utf8_lossy(&result.stderr)
                );
                return Vec::new(); // Return an empty vector on failure
            }
            // Return the raw bytes of the image from stdout
            result.stdout
        }
        Err(err) => {
            log::error!("thumbnail_extraction:: Failed to execute ffmpeg: {}", err);
            Vec::new()
        }
    }
}
